/*****************************************************************************
 *
 * utouch-frame - Touch Frame Library
 *
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#include "v2/device.h"

#include <assert.h>

#include "v2/axis.h"

namespace utouch {
namespace frame {

UFStatus UFDevice::GetAxisByIndex(unsigned int index, ::UFAxis* axis) const {
  if (index >= axes_.size())
    return UFStatusErrorInvalidAxis;

  auto it = axes_.cbegin();
  std::advance(it, index);

  *axis = it->second.get();

  return UFStatusSuccess;
}

UFStatus UFDevice::GetAxisByType(UFAxisType type, ::UFAxis* axis) const {
  auto it = axes_.find(type);
  if (it == axes_.end())
    return UFStatusErrorInvalidAxis;

  *axis = it->second.get();

  return UFStatusSuccess;
}

} // namespace frame
} // namespace utouch

extern "C" {

UFStatus frame_device_get_property_string_(UFDevice device,
                                           UFDeviceProperty property,
                                           char **value) {
  return static_cast<const utouch::frame::UFDevice*>(device)->GetProperty(
      property,
      value);
}

UFStatus frame_device_get_property_int_(UFDevice device,
                                        UFDeviceProperty property, int *value) {
  return static_cast<const utouch::frame::UFDevice*>(device)->GetProperty(
      property,
      value);
}

UFStatus frame_device_get_property_unsigned_int_(UFDevice device,
                                                 UFDeviceProperty property,
                                                 unsigned int *value) {
  return static_cast<const utouch::frame::UFDevice*>(device)->GetProperty(
      property,
      value);
}

#undef frame_device_get_property /* Override C11 generic selections macro */
UFStatus frame_device_get_property(UFDevice device, UFDeviceProperty property,
                                   void *value) {
  return static_cast<const utouch::frame::UFDevice*>(device)->GetProperty(
      property,
      value);
}

UFStatus frame_device_get_axis_by_index(UFDevice device, unsigned int index,
                                        UFAxis* axis) {
  return static_cast<const utouch::frame::UFDevice*>(device)->GetAxisByIndex(
      index,
      axis);
}

UFStatus frame_device_get_axis_by_type(UFDevice device, UFAxisType type,
                                       UFAxis* axis) {
  return static_cast<const utouch::frame::UFDevice*>(device)->GetAxisByType(
      type,
      axis);
}

unsigned int frame_device_get_num_axes(UFDevice device) {
  unsigned int num_axes;
  UFStatus status = frame_device_get_property(device, UFDevicePropertyNumAxes,
                                              &num_axes);
  assert(status == UFStatusSuccess);
  return num_axes;
}

} // extern "C"
