/*****************************************************************************
 *
 * utouch-frame - Touch Frame Library
 *
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#ifndef UTOUCH_FRAME_DEVICE_X11_H_
#define UTOUCH_FRAME_DEVICE_X11_H_

#include <map>

#include <X11/Xlib.h>
#include <X11/extensions/XInput2.h>

#include "utouch/frame.h"
#include "v2/device.h"
#include "v2/typedefs.h"

namespace utouch {
namespace frame {

class UFDeviceX11
    : public UFDevice,
      public std::enable_shared_from_this<UFDeviceX11> {
 public:
  UFDeviceX11(Display* display, const XIDeviceInfo& info);

  int master_id() const { return master_id_; }
  const std::map<int, UFAxisType>& axis_map() const { return axis_map_; }

  bool HandleDeviceEvent(const XIDeviceEvent* event, SharedUFFrame* frame);
  bool HandleOwnershipEvent(const XITouchOwnershipEvent* event,
                            SharedUFFrame* frame);
  UFStatus AcceptTouch(::Window window_id, UFTouchId touch_id) const;
  UFStatus RejectTouch(::Window window_id, UFTouchId touch_id) const;
  void ReleaseFrames();

  UFDeviceX11(const UFDeviceX11&) = delete;
  UFDeviceX11& operator=(const UFDeviceX11&) = delete;

 private:
  Display* display_;
  int master_id_;
  std::map<int, UFAxisType> axis_map_;
  std::map< ::Window, SharedWindow> windows_;
};

} // namespace frame
} // namespace utouch

#endif // UTOUCH_FRAME_DEVICE_X11_H_
